@@ -38,9 +38,10 @@ shinyServer(function(input, output, session) {
        Name = c("Prod1","Prod2","Prod3","Prod4"),
        'Pre-merger\n Owner'  = c("Firm1","Firm2","Firm3","Firm3"),
        'Post-merger\n Owner' = c("Firm1","Firm1","Firm3","Firm3"),
-       'Prices \n($/unit)'    =c(.0441,.0328,.0409,.0396)*100,
-       'Quantity Shares'   =c(0.09734513, 0.25368732, 0.37315634, 0.27581121),
-       'Margins\n(proportion)' =c(NA,.5515,.5421,.49),
+       'Prices \n($/unit)'    = rep(10,4),
+       'Quantities'   =c(0.5,.25,.15,.1)*100,
+       'Margins\n(p-c)/p' =c(0.5,NA,NA,NA),
+       'Post-merger\n Cost Changes\n(Proportion)' = rep(0,4),
        stringsAsFactors = FALSE,
        check.names=FALSE
      )
@@ -52,6 +53,7 @@ shinyServer(function(input, output, session) {
      #if(input$incEff) inputData$mcDelta <- 0
      
      inputData[(nDefProd+1):nPossProds,] <- NA
+     inputData <- inputData[order(inputData$`Quantities`, decreasing = TRUE),]
      rownames(inputData) <- NULL
      
     
@@ -66,10 +68,11 @@ shinyServer(function(input, output, session) {
      isCournot <- grepl("Cournot",class(res))
      isAuction <- grepl("Auction",class(res))
      isRevDemand <- grepl("ces|aids",class(res),ignore.case = TRUE)
+     missPrices <- any(is.na(res@prices))
      
      inLevels <- FALSE
      
-     if(isAuction && any(is.na(res@prices))){inLevels = TRUE}
+     if(isAuction && missPrices){inLevels = TRUE}
 
      
      
@@ -82,8 +85,9 @@ shinyServer(function(input, output, session) {
      }
      
      else{
-       capture.output(s <- summary(res, revenue = isRevDemand,levels = inLevels))
-       theseshares <- res@shares
+       capture.output(s <- summary(res, revenue = isRevDemand & missPrices,levels = inLevels))
+       theseshares <- calcShares(res, revenue=isRevDemand & missPrices)
+       theseshares <- theseshares/sum(theseshares)
        }
      
      isparty <- s$isParty == "*"
@@ -100,6 +104,7 @@ shinyServer(function(input, output, session) {
      
      res <- with(s, data.frame(
            #'HHI Change' = as.integer(round(hhi(res,preMerger=FALSE) -  hhi(res,preMerger=TRUE))),
+           'Pre-Merger HHI' =  as.integer(HHI(theseshares,owner=res@ownerPre)),
            'HHI Change' = as.integer(HHI(theseshares,owner=res@ownerPost) - HHI(theseshares,owner=res@ownerPre)),
            'Industry Price Change (%)' = sum(priceDelta * sharesPost/100,na.rm=TRUE),
            'Merging Party Price Change (%)'= sum(priceDelta[isparty] * partyshare, na.rm=TRUE) / sum(partyshare),
@@ -142,7 +147,9 @@ shinyServer(function(input, output, session) {
     obsMargins <- res@margins
     obsElast <- res@mktElast
     
-    if(length(obsMargins[!is.na(obsMargins)]) <= 2){return()}
+    #if(length(obsMargins[!is.na(obsMargins)]) < 2){return()}
+    
+    
     prePrices <- unname(drop(res@pricePre))
     preMargins <- drop(calcMargins(res, preMerger=TRUE))
     preShares <- drop(calcShares(res, preMerger=TRUE))
@@ -159,13 +166,13 @@ shinyServer(function(input, output, session) {
       check.names = FALSE
     )*100
     
-    rmThese <- colSums(abs(res),na.rm=TRUE)
+    #rmThese <- colSums(abs(res),na.rm=TRUE)
     
   
    
     if(isCournot)  res[-1,'Prices'] <- NA
      
-    res <- res[,rmThese >1e-3,drop=FALSE]
+    #res <- res[,rmThese >1e-3,drop=FALSE]
     
     
     if(!isCournot) rownames(res) <- labels
@@ -175,7 +182,7 @@ shinyServer(function(input, output, session) {
     else{ res <- data.frame('Market Elasticity'= 1 - obsElast/preElast,
                             check.names = FALSE)*100
     
-    if(res < 1e-3) res <- NULL
+    #if(res < 1e-3) res <- NULL
     }
     return(res)
    }
@@ -300,15 +307,16 @@ shinyServer(function(input, output, session) {
    
    
    
-   ## create a reactive object that tracks which demand is being used
-   demand <- eventReactive(input$simulate, {ifelse(input$supply =="Bertrand", 
-
-                                              ifelse(input$calcElast == "2 or more margins",input$demand_bert_alm,input$demand_bert),
-                                                      ifelse(input$supply == "Cournot",
-                                                     ifelse(input$calcElast == "2 or more margins",input$demand_cournot_alm, input$demand_cournot),
-                                                     ifelse(input$calcElast == "2 or more margins",input$demand_2nd_alm, input$demand_2nd)))
-                      })
-   
+   # ## create a reactive object that tracks which demand is being used
+   # demand <- eventReactive(input$simulate, {
+   #                                         ifelse(input$supply =="Bertrand", 
+   # 
+   #                                            ifelse(input$calcElast == "2 or more margins",input$demand_bert_alm,input$demand_bert),
+   #                                                    ifelse(input$supply == "Cournot",
+   #                                                   ifelse(input$calcElast == "1 or more margins",input$demand_cournot_alm, input$demand_cournot),
+   #                                                   ifelse(input$calcElast == "2 or more margins",input$demand_2nd_alm, input$demand_2nd)))
+   #                    })
+   # 
    
    ## create a reactive list of objects
    values <- reactiveValues(inputData = genInputData(), sim = NULL, msg = NULL)
@@ -318,6 +326,47 @@ shinyServer(function(input, output, session) {
    ## update reactive list whenever changes are made to input
     observe({
     
+      supply <- input$supply
+      demand <- gsub("\\s*(unknown elasticity)","",input$demand,perl=TRUE)
+      
+      provElast <- grepl('elasticity',input$calcElast)
+      defElast <- ifelse(provElast, 1 , 2)
+      
+     if(supply == 'Cournot'){
+       theseChoices <- c("market elasticity and 0 or more margins",
+                         "1 or more margins"
+       )
+       
+       
+       demandChoices<- c("linear","log")
+       
+       }
+      
+    else{
+      theseChoices <- c("market elasticity and 1 or more margins",
+                        "2 or more margins"
+      )
+      updateRadioButtons(session = session, "calcElast", "Calibrate model parameters using:",
+                         choices = theseChoices , selected = theseChoices[defElast]) 
+      
+      if(supply == 'Bertrand'){demandChoices<- c('logit','ces','aids')}
+      else{demandChoices<- c('logit')}
+      
+     
+      
+    }
+      
+      if( !provElast ){ demandChoices <- paste(demandChoices,"(unknown elasticity)")}
+      
+      updateRadioButtons(session = session, "calcElast", "Calibrate model parameters using:",
+                         choices = theseChoices , selected = theseChoices[defElast])
+      
+      provDemand <- grep(demand, demandChoices)
+      provDemand <- ifelse(length(provDemand) > 0 , provDemand, 1)
+      
+      updateSelectInput(session=session, "demand", "Demand Specification:",
+                        choices = demandChoices, selected = demandChoices[provDemand])
+      
       
       if(!is.null(input$hot)){
         values$inputData = hot_to_r(input$hot)
@@ -332,19 +381,19 @@ shinyServer(function(input, output, session) {
       inputData <- values[["inputData"]]
       
       prices <- inputData[,"Prices \n($/unit)"]
-      shares <- inputData[,grepl("Shares",colnames(inputData))]
+      output <- inputData[,grepl("Quantities|Revenue",colnames(inputData), perl=TRUE)]
       
-      missPrices <- isTRUE(any(is.na(prices[ !is.na(shares) ] ) ))
+      missPrices <- isTRUE(any(is.na(prices[ !is.na(output) ] ) ))
       
       if(missPrices && input$supply =="2nd Score Auction"){colnames(inputData)[grepl("Margins",colnames(inputData))] <- "Margins\n ($/unit)"}
-      else{colnames(inputData)[grepl("Margins",colnames(inputData))] <- "Margins\n (proportion)"}
-      if(input$supply =="Cournot"){colnames(inputData)[grepl("Shares",colnames(inputData))] <- "Quantities"}
-      else if (missPrices && any(grepl("ces|aids",c(input$demand_bert_alm,input$demand_bert,input$demand_2nd_alm, input$demand_2nd), perl=TRUE), na.rm=TRUE)){colnames(inputData)[grepl("Shares",colnames(inputData))] <- "Revenue\nShares"}
-      else{{colnames(inputData)[grepl("Quant|Shares",colnames(inputData))] <- "Quantity\nShares"}}
+      else{colnames(inputData)[grepl("Margins",colnames(inputData))] <- "Margins\n (p-c)/p"}
+
+      if (missPrices && any(grepl("ces|aids",input$demand, perl=TRUE), na.rm=TRUE)){colnames(inputData)[grepl("Quantities",colnames(inputData))] <- "Revenues"}
+      else{{colnames(inputData)[grepl("Revenues",colnames(inputData))] <- "Quantities"}}
       
       if (!is.null(inputData))
-        rhandsontable(inputData, stretchH = "all") %>% hot_col(col = 1:ncol(inputData), valign = "htMiddle") %>%
-        hot_col(col = which (sapply(inputData,is.numeric)),halign = "htCenter" )
+        rhandsontable(inputData, stretchH = "all", contextMenu = FALSE ) %>% hot_col(col = 1:ncol(inputData), valign = "htMiddle") %>%
+        hot_col(col = which (sapply(inputData,is.numeric)),halign = "htCenter" ) %>% hot_cols(columnSorting = TRUE)
     })
 
 
@@ -359,18 +408,19 @@ shinyServer(function(input, output, session) {
       
       indata <- values[["inputData"]]
       
-      isShares <-  grepl("Quantities|Shares",colnames(indata),perl = TRUE)
-     
+      isOutput <-  grepl("Quantities|Revenues",colnames(indata),perl = TRUE)
+      isCournot <- input$supply == "Cournot"
+      
       colnames(indata)[ grepl("Margins",colnames(indata),perl = TRUE)] <- "Margins"
       
-      colnames(indata)[isShares] <- "Output"
+      colnames(indata)[isOutput] <- "Output"
       
       indata <- indata[!is.na(indata[,"Output"]),]
       
-      if(any(isShares)) indata$Output <- indata$Output/sum(indata$Output, na.rm=TRUE) ## normalize shares to sum to 1
+      if(!isCournot) indata$Output <- indata$Output/sum(indata$Output, na.rm=TRUE) ## normalize shares to sum to 1
       
       
-      indata$mcDelta <- 0
+      indata$mcDelta <- indata$'Post-merger\n Cost Changes\n(Proportion)'
       
       indata$'Pre-merger\n Owner' <- factor(indata$'Pre-merger\n Owner')
       indata$'Post-merger\n Owner' <- factor(indata$'Post-merger\n Owner',levels=levels(indata$'Pre-merger\n Owner'))
@@ -378,7 +428,7 @@ shinyServer(function(input, output, session) {
       
      
       thisSim <- msgCatcher(
-                          runSims(supply = input$supply,demand = demand(), indata = indata, mktElast = input$enterElast )
+                          runSims(supply = input$supply,demand = input$demand, indata = indata, mktElast = input$enterElast )
       )
 
     
@@ -434,17 +484,23 @@ shinyServer(function(input, output, session) {
           missPrice <- any(is.na(values[["sim"]]@prices)) 
           if(isAuction && missPrice){inLevels = TRUE}
           
-          capture.output(res <- summary(values[["sim"]], revenue=isRevDemand, levels=inLevels))
+          capture.output(res <- summary(values[["sim"]], revenue=isRevDemand & missPrice, insideOnly=TRUE, levels=inLevels))
           res$Name <- rownames(res)
+          res$mcDelta <- NULL
           res <- res[,c(1, ncol(res), 2 : (ncol(res)  - 1))]
+          res$cmcr <- NA
+          try(res$cmcr[res$isParty=="*"] <- cmcr(values[["sim"]]))
+          
+          thesenames <-  c("Merging Party","Name","Pre-Merger Price","Post-Merger Price", "Price Change (%)","Pre-Merger Share (%)","Post-Merger Share (%)", "Share Change (%)",'Compensating Marginal Cost Reduction (%)')
           
-          thesenames <-  c("Merging Party","Name","Pre-Merger Price","Post-Merger Price", "Price Change (%)","Pre-Merger Share (%)","Post-Merger Share (%)", "Share Change (%)")
           
           #if(isAIDS && missPrice){thesenames <- thesenames[!thesenames %in% c("Pre-Merger Price","Post-Merger Price")]}
           
           colnames(res) <- thesenames
           
-          res[,c("Pre-Merger Share (%)","Post-Merger Share (%)")] <-  res[,c("Pre-Merger Share (%)","Post-Merger Share (%)")] * 100 / colSums( res[,c("Pre-Merger Share (%)","Post-Merger Share (%)")])
+          if(all(is.na(res$`Compensating Marginal Cost Reduction (%)`))) res$`Compensating Marginal Cost Reduction (%)` <- NULL
+          
+          #res[,c("Pre-Merger Share (%)","Post-Merger Share (%)")] <-  res[,c("Pre-Merger Share (%)","Post-Merger Share (%)")] * 100 / colSums( res[,c("Pre-Merger Share (%)","Post-Merger Share (%)")])
         
           if(inLevels){ colnames(res)[ colnames(res) == "Price Change (%)"] = "Price Change ($/unit)"}
           
@@ -454,7 +510,7 @@ shinyServer(function(input, output, session) {
         res
         
       
-      }, digits = 1)  
+      }, digits = 2)  
     
     
     output$results_shareOut <- renderTable({
@@ -519,10 +575,17 @@ shinyServer(function(input, output, session) {
       
       if(input$inTabset!= "elastpanel" || input$simulate == 0 || is.null(values[["sim"]])){return()}
         
+      isCournot <- grepl("Cournot",class(values[["sim"]]))
+      
        if(input$pre_elast == "Pre-Merger"){ preMerger = TRUE}
         else{preMerger =FALSE}
         
-        res <- elast(values[["sim"]], preMerger=preMerger)
+      if(!isCournot && input$diversions){
+        res <- diversion(values[["sim"]], preMerger=preMerger)
+      }
+      else{  res <- elast(values[["sim"]], preMerger=preMerger)}
+        if(isCournot){colnames(res) <- "Elasticity"}
+        
         res
         
       
@@ -543,6 +606,93 @@ shinyServer(function(input, output, session) {
     }, rownames = FALSE)   
 
 
+    ## display R code to code tab
+    output$results_code <- renderPrint({
+      
+      if(input$inTabset!= "codepanel"){return()}  
+      
+      indata <- values[["inputData"]]
+      indata <- indata[!is.na(indata$Name),]
+      
+      cnames <- colnames(indata)
+      cnames <- gsub("\n","",cnames)
+      
+      firstPrice <- which(!is.na(indata[,grep("price",cnames, ignore.case = TRUE)]))[1]
+      
+      argnames <- c("demand",
+                    "ownerPre",
+                    "ownerPost",
+                    "prices",
+                    "quantities",
+                    "margins",
+                    "mcDelta")
+      
+      argvalues   <- paste0(argnames," = simdata$`",cnames,"`")
+  
+    thisElast <- ifelse(grepl("elast",input$calcElast), 
+                        input$enterElast,
+                        NA
+    )
+    
+    thisdemand <- gsub("\\s*\\(.*","",input$demand,perl=TRUE)
+      
+      argvalues[1] <- paste(c("demand = ", shQuote(thisdemand)), collapse = "") 
+     
+      argvalues <- c(argvalues,
+                     paste0("mktElast = ", thisElast,collapse = ""),
+                     "labels = simdata$Name"
+      )
+      
+      
+      
+      
+      if(input$supply == "Cournot"){
+        atrfun <- "cournot"
+        argvalues[grep("prices", argvalues)] <- paste0(argvalues[grep("prices", argvalues)],"[",firstPrice,"]")
+        argvalues[grep("quantities", argvalues)] <- "quantities = as.matrix(simdata$`Quantities`)" 
+        argvalues[grep("margins", argvalues)] <- paste0("margins = as.matrix(simdata$`",grep("Margin",cnames,value = TRUE),"`)")
+
+        argvalues[grep("labels", argvalues)] <- sprintf("labels = list(as.character(simdata$Name),as.character(simdata$Name[%d]))",firstPrice) 
+}
+      else if( input$supply =="Bertrand"){atrfun <- "bertrand.alm"}
+      else{atrfun <- "auction2nd.logit.alm"
+           argvalues <- argvalues[-1]
+           argvalues[grep("quantities", argvalues)] <- "shares = simdata$`Quantities` / sum( simdata$`Quantities` ) "
+           argvalues[grep("margins", argvalues)] <- paste0("margins = simdata$`",
+                                                           grep("Margin",cnames,value = TRUE),
+                                                            "` * ", "simdata$`", grep("Price", cnames, value=TRUE),"`")
+           }
+      
+      atrfun <- paste0("simres <- ",atrfun,"(\n\t",paste0(argvalues,collapse = ",\n\t"),")",collapse = "\n")
+      
+      indata_code <- sapply(1:ncol(indata),
+                            function(x){d <- indata[,x];
+                                      if(is.character(d)){d <- sprintf("'%s'", indata[,x])};
+                                      paste0('`',cnames[x],'`',"= c(",paste(d,collapse=","),")")}
+                            )
+      indata_code <- paste0("simdata <- data.frame(\n\t", paste0(indata_code,collapse=",\n\t"),",\n check.names = FALSE,\n stringsAsFactors = FALSE\n)")
+      
+      thiscode <- c(
+        "library(antitrust)",
+        "\n\n ## Load Data:\n",
+        indata_code,
+        "\n\n ## Run Simulation: \n",
+        atrfun,
+        "\n\n ## Summary Tab Results:\n",
+        "summary(simres, levels = FALSE, market=TRUE)",
+        "\n\n ## Details Tab Results:\n",
+        "summary(simres, levels = FALSE, market=FALSE)\n\n",
+        "\n\n ## Elasticities Tab Results  (Pre-merger Only):\n",
+        "elast(simres, preMerger = TRUE, market=TRUE)\n elast(simres, preMerger = TRUE, market=FALSE)",
+        "\n\n ## Diagnostics Tab Results:\n",
+        "calcDiagnostics(simres)\n\n"
+      )
+      
+      cat(thiscode)
+      
+      
+    })  
+    
     ## display messages to message tab
    output$warnings <- renderPrint({
     
